/* command.c - command infrastructure
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#define COMMAND_C
#include <config.h>
#include <pthread.h>
#include <com_err.h>
#include "command.h"
#include "../daemon/daemon.h"

static unsigned command_handler_concurrency
= COMMAND_HANDLER_DEFAULT_CONCURRENCY;

static dispatch_queue_t *command_handler_reader_queue = NULL;

static dispatch_queue_t *command_handler_writer_queue = NULL;

static pthread_once_t command_handler_queue_once = PTHREAD_ONCE_INIT;

static void
command_handler_queue_init(void)
{
  result_t result;

  result = dispatch_queue_alloc(&command_handler_reader_queue,
				command_handler_concurrency);

  if (result == OK)
    result = dispatch_queue_alloc(&command_handler_writer_queue, 1);

  if (result != OK)
    daemon_fatal(result, "Initializing command handler queue");
}

result_t
command_set_concurrency(unsigned concurrency)
{
  command_handler_concurrency = concurrency;
  return OK;
}

result_t
command_get_reader_queue(dispatch_queue_t ** queue)
{
  I(queue);
  I(!*queue);

  pthread_once(&command_handler_queue_once,
	       &command_handler_queue_init);

  *queue = command_handler_reader_queue;
  return OK;
}

result_t
command_get_writer_queue(dispatch_queue_t ** queue)
{
  I(queue);
  I(!*queue);

  pthread_once(&command_handler_queue_once,
	       &command_handler_queue_init);

  *queue = command_handler_writer_queue;
  return OK;
}

void
command_shutdown(void)
{
  pthread_once(&command_handler_queue_once,
	       &command_handler_queue_init);
  dispatch_queue_shutdown(command_handler_writer_queue);
  dispatch_queue_shutdown(command_handler_reader_queue);
}
