/* criteria.h--criteria interfaces
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef CRITERIA_H
#define CRITERIA_H

#include "vent.h"
#include "../util/sllist.h"

struct comparator_s;

typedef struct criterion_s criterion_t;

/* The slightly wonky order here is because I'm grouping by
 * the type of the criterion, not alphabetically. */
typedef enum {
  CRITERION_AND,
  CRITERION_OR,
  CRITERION_NOT,
  CRITERION_ALL,
  CRITERION_RANGE,
  CRITERION_COMPARE,
  CRITERION_COMPARESTRICT,
  CRITERION_EQUAL,
  CRITERION_PREFIX,
  CRITERION_SUBSTRING
}               criterion_tag_t;

extern          result_t
_criterion_conjunction_alloc(criterion_tag_t tag,
			     criterion_t * first,
			     criterion_t * second,
			     criterion_t ** dest);

#define criterion_and_alloc(f, s, d)			\
	(_criterion_conjunction_alloc(CRITERION_AND,	\
				      (f), (s), (d)))

#define criterion_or_alloc(f, s, d)			\
	(_criterion_conjunction_alloc(CRITERION_OR,	\
				      (f), (s), (d)))

#define criterion_not_alloc(f, d)			\
	(_criterion_conjunction_alloc(CRITERION_NOT,	\
				      (f), NULL, (d)))

#define criterion_all_alloc(d)				\
	(_criterion_conjunction_alloc(CRITERION_ALL,	\
				      NULL, NULL, (d)))

extern result_t
criterion_range_alloc(unsigned low,
		      unsigned high,
		      str_t * modtime,
		      criterion_t ** dest);

extern result_t
_criterion_comparison_alloc(criterion_tag_t tag,
			    str_t * attribute,
			    str_t * comparator,
			    str_t * value,
			    criterion_t ** dest);

#define criterion_compare_alloc(a, c, v, d)			\
	(_criterion_comparison_alloc(CRITERION_COMPARE,		\
				     (a), (c), (v), (d)))

#define criterion_comparestrict_alloc(a, c, v, d)		\
	(_criterion_comparison_alloc(CRITERION_COMPARESTRICT,	\
				     (a), (c), (v), (d)))

#define criterion_equal_alloc(a, c, v, d)			\
	(_criterion_comparison_alloc(CRITERION_EQUAL,		\
				     (a), (c), (v), (d)))

#define criterion_prefix_alloc(a, c, v, d)			\
	(_criterion_comparison_alloc(CRITERION_PREFIX,		\
				     (a), (c), (v), (d)))

#define criterion_substring_alloc(a, c, v, d)			\
	(_criterion_comparison_alloc(CRITERION_SUBSTRING,	\
				     (a), (c), (v), (d)))

extern result_t criterion_free(criterion_t ** criterion);

/* criterion_eval actually evaluates a criterion, checking to see
 * whether some entry matches it.  The index is the entry's context
 * index, and should be zero if this isn't being used as part of an
 * enumerated context.  The stack is used in the process of evaluating
 * the criterion; it should initially point to a NULL sllist_t,
 * and may be reused in subsequent criterion_eval loops, and eventually
 * freed via sllist_free(). */

extern result_t
criterion_eval(criterion_t * criterion,
	       sllist_t ** stack,
	       const vent_t * entry,
	       unsigned index,
	       bool_t * result);

#endif				/* CRITERIA_H */
