/* search_ast.h--search AST definitions
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef SEARCH_AST_H
#define SEARCH_AST_H

#include "comparator.h"

struct client_s;

typedef struct search_return_metadatum_s {
  struct search_return_metadatum_s *next;
  int             name;
  str_t          *name_str;
}               search_return_metadatum_t;

#define search_ret_meta_NEXT(m) ((m)->next)
#define search_ret_meta_NAME(m) ((m)->name)

typedef struct search_return_s {
  struct search_return_s *next;
  str_t          *attribute;
  search_return_metadatum_t *metadata;
}               search_return_t;

#define search_return_NEXT(r) ((r)->next)
#define search_return_ATTR(r) ((r)->attribute)
#define search_return_METADATA(r) ((r)->metadata)

typedef struct search_sort_s {
  struct search_sort_s *next;
  str_t          *attribute;
  compare_ordering_proc_t *ordering;
  str_t          *ordering_str;
}               search_sort_t;

#define search_sort_NEXT(s) ((s)->next)
#define search_sort_ATTR(s) ((s)->attribute)
#define search_sort_ORDERING(s) ((s)->ordering)

typedef struct search_modifier_s {
  unsigned        depth,
                  hardlimit,
                  limit,
                  limit_ret;
  bool_t          enumerate,
                  notify;
  str_t          *context;
  search_return_t *ret;
  search_sort_t  *sort;

  bool_t          got_depth,
                  got_hard,
                  got_limit,
                  got_context,
                  got_noinherit,
                  got_return,
                  got_sort;
}               search_modifier_t;

#define search_modifier_DEPTH(m) ((m)->depth)
#define search_modifier_GOT_DEPTH(m) ((m)->got_depth)
#define search_modifier_HARDLIMIT(m) ((m)->hardlimit)
#define search_modifier_LIMIT(m) ((m)->limit)
#define search_modifier_LIMIT_RET(m) ((m)->limit_ret)
#define search_modifier_CONTEXT(m) ((m)->context)
#define search_modifier_ENUMERATE(m) ((m)->enumerate)
#define search_modifier_NOTIFY(m) ((m)->notify)
#define search_modifier_NOINHERIT(m) ((m)->got_noinherit)
#define search_modifier_RETURN(m) ((m)->ret)
#define search_modifier_SORT(m) ((m)->sort)

extern          result_t
search_return_metadatum_alloc(struct client_s * client,
			      str_t * metadatum,
			      search_return_metadatum_t ** dest);

#define search_return_metadatum_list_append(l, m, d)	\
	((m)->next = (l), *(d) = (m), OK)

extern result_t
search_return_alloc(struct client_s * client,
		    str_t * attribute,
		    search_return_metadatum_t * metadata,
		    search_return_t ** dest);

#define search_return_list_append(l, r, d)	\
	((r)->next = (l), *(d) = (r), OK)

extern result_t
search_sort_alloc(struct client_s * client,
		  str_t * attribute,
		  str_t * ordering,
		  search_sort_t ** dest);

#define search_sort_list_append(l, s, d)	\
	((s)->next = (l), *(d) = (s), OK)

extern result_t
search_modifier_alloc(struct client_s * client,
		      search_modifier_t ** dest);

extern result_t search_modifier_free(search_modifier_t ** mod);

#define search_modifier_set_depth(m, c, d)			\
	((m)->got_depth ? (client_SET_BAD(c),			\
			   ACAP_ERR_DUPLICATE_DEPTH_MODIFIER)	\
			: ((m)->depth = (d),			\
			   (m)->got_depth = TRUE,		\
			   OK))

#define search_modifier_set_hardlimit(m, c, l)				\
	((m)->got_hard ? (client_SET_BAD(c),				\
			  ACAP_ERR_DUPLICATE_HARDLIMIT_MODIFIER)	\
		       : ((m)->hardlimit = (l),				\
	 		  (m)->got_hard = TRUE,				\
	 		  OK))

#define search_modifier_set_limit(m, c, l, h)			\
	((m)->got_limit ? (client_SET_BAD(c),			\
			   ACAP_ERR_DUPLICATE_LIMIT_MODIFIER)	\
			: ((m)->limit = (l),			\
			   (m)->limit_ret = (h),		\
	 		   (m)->got_limit = TRUE,		\
	 		   OK))

#define search_modifier_set_makecontext(m, c, e, n, x)			\
	((m)->got_context ? (client_SET_BAD(c),				\
			     ACAP_ERR_DUPLICATE_MAKECONTEXT_MODIFIER)	\
			  : ((m)->enumerate = (e),			\
			     (m)->notify = (n),				\
			     (m)->context = (x),			\
	 		     (m)->got_context = TRUE,			\
	 		     OK))

#define search_modifier_set_noinherit(m, c)				\
	((m)->got_noinherit ? (client_SET_BAD(c),			\
			       ACAP_ERR_DUPLICATE_NOINHERIT_MODIFIER)	\
			    : ((m)->got_noinherit = TRUE,		\
	 		       OK))

#define search_modifier_set_return(m, c, r)				\
	((m)->got_return ? (client_SET_BAD(c),				\
			    ACAP_ERR_DUPLICATE_RETURN_MODIFIER)		\
			 : ((m)->ret = (r),				\
			    sllist_reverse((sllist_t **)&(m)->ret),	\
	 		    (m)->got_return = TRUE,			\
	 		    OK))

#define search_modifier_set_sort(m, c, s)				\
	((m)->got_sort ? (client_SET_BAD(c),				\
			  ACAP_ERR_DUPLICATE_SORT_MODIFIER)		\
		       : ((m)->sort = (s),				\
			  sllist_reverse((sllist_t **)&(m)->sort),	\
	 		  (m)->got_sort = TRUE,				\
	 		  OK))
#endif				/* SEARCH_AST_H */
