/* vent_vec.h--virtual entry vector interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef VENT_VEC_H
#define VENT_VEC_H

/* The idea is to provide an abstract interface to a vector of
 * entries, such as might be contained in a context, returned to a
 * user, etc.  It's a container of vent_t structures, each of which
 * refers to a single virtual entry in the result. */

#include "vent.h"

typedef struct {
  enum {
    VEC_UNINIT,
    VEC_ADDING,
    VEC_INITED
  }               state;
  uint32_t        n_entries;
  vent_t         *entries;
}               vent_vec_t;

#define vent_vec_SIZE(v) ((v)->n_entries)

extern          result_t vent_vec_alloc(vent_vec_t ** dest);

extern result_t
vent_vec_dup(vent_vec_t * src,
	     vent_vec_t ** dest);

extern result_t vent_vec_free(vent_vec_t ** vec);

typedef struct {
  unsigned        entries_size;
}               vent_vec_add_state_t;

extern result_t
vent_vec_begin_additions(vent_vec_t * vec,
			 vent_vec_add_state_t * state);

/* This takes ownership of the data in vent--the name, and the entries. */
extern result_t
vent_vec_add(vent_vec_t * vec,
	     vent_vec_add_state_t * state,
	     vent_t * vent);

extern result_t
vent_vec_end_additions(vent_vec_t * vec,
		       vent_vec_add_state_t * state);

typedef result_t
vent_compare_proc_t(const vent_t * a,
		    const vent_t * b,
		    void *rock,
		    compare_result_t * compare_result);

extern result_t
vent_vec_sort(vent_vec_t * vec,
	      vent_compare_proc_t * comparator,
	      void *rock);

/* And for traversal... */
typedef struct {
  vent_t         *it,
                 *end;
}               vent_vec_iterator_t;

#define vent_vec_ITER_BEGIN(v, i) ((i)->it = (v)->entries,		   \
				   (i)->end = (v)->entries + (v)->n_entries)
#define vent_vec_ITER_ATEND(i) ((i)->end <= (i)->it)
#define vent_vec_ITER_INCR(i) ((i)->it++)
#define vent_vec_ITER_VALUE(i) ((i)->it)
#endif				/* VENT_VEC_H */
