/* acconfig.h - autoheader input and global acapd definitions
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef CONFIG_H
#define CONFIG_H
/*INDENT OFF*/
@TOP@
/*INDENT ON*/

/* Our package */
#undef PACKAGE

/* Our version */
#undef VERSION

/* Root ACAP data directory */
#undef ACAPD_ROOT

/* _POSIX_PTHREAD_SEMANTICS needed? */
#undef _POSIX_PTHREAD_SEMANTICS

/* _REENTRANT needed? */
#undef _REENTRANT

/* ssize_t */
#undef ssize_t

/* Do we have thr_yield? */
#undef HAVE_THR_YIELD

/* check for integer types... */
#undef HAVE_INTTYPES_H

/* int types--note that if inttypes.h is available,
 * it will be used, and these will be meaningless. */
#undef CMU_HAVE_INT8_T
#undef CMU_HAVE_UINT8_T
#undef CMU_HAVE_U_INT8_T
#undef CMU_HAVE_INT16_T
#undef CMU_HAVE_UINT16_T
#undef CMU_HAVE_U_INT16_T
#undef CMU_HAVE_INT32_T
#undef CMU_HAVE_UINT32_T
#undef CMU_HAVE_U_INT32_T

/* _SGI_REENTRANT_FUNCTIONS needed? */
#undef _SGI_REENTRANT_FUNCTIONS

/* This seems to be required to make Solaris happy. */
#undef __EXTENSIONS__

/* NANA check */
#undef WITHOUT_NANA
/*INDENT OFF*/
@BOTTOM@
/*INDENT ON*/

#define L_DEFAULT_GUARD (debug_all)
#define L_DEFAULT_HANDLER acapd_log
#define I_DEFAULT_HANDLER(expr,file,line,param) \
	acapd_failed_assertion(expr,file,line)

#include <util/util.h>

#ifndef WITHOUT_NANA
#include <stdio.h>

extern void     acapd_log(FILE * file, const char *format,...);

extern void
acapd_failed_assertion(const char *expression,
		       const char *file,
		       unsigned line) __attribute__((noreturn));

#endif				/* ! WITHOUT_NANA */

/* concurrency levels */
#define CONN_HANDLER_DEFAULT_CONCURRENCY 2
/* Until DB behaves nicer, this should remain... low. */
#define COMMAND_HANDLER_DEFAULT_CONCURRENCY 1

/* caches */
#define DATASET_REF_DEFAULT_CACHE_SIZE 50
#define DATASET_DATA_DEFAULT_CACHE_SIZE 10

#define ACAP_DEFAULT_PORT_NAME "acap"
#define ACAP_DEFAULT_PORT_NUMBER 674

#define GETSERVBYNAME_R_BUF_SZ 64	/* buffer size to use with
					 * getservbyname_r */

#ifndef __GNUC__
/* We can't use gcc attributes; define them away... */
#define __attribute__(foo)
#endif				/* USE_GCC_ATTRIBUTES */

  extern int      debug_all,
                  debug_acapd_io,
                  acap_debug,	/* used by acap parser */
                  debug_daemon_support,
                  debug_daemon_rselock,
                  debug_daemon_io,
                  debug_acapd_interlock,
                  debug_acapd_search,
                  debug_acapd_dataset,
                  debug_acapd_store,
                  debug_acapd_data,
                  debug_acapd_context,
                  debug_acapd_client,
                  debug_acapd_dataset_db;

/* Our database environment */
  extern struct __db_env *acapd_db_env;

#ifdef sgi
  typedef void   *spcb_p;

  typedef void   *ptcb_p;

#endif

  extern const char *program_name;	/* set to be the name of the program */

#define ACAP_MAX_QSTR_LEN 1024	/* from the spec */

#define CONN_BUFFER_SIZE 1024	/* amount of data we'll try to buffer */

/* ACAP data */
#define ACAPD_DATA_ROOT ACAPD_ROOT "/data"

/* Maximium inheritance depth (maybe this should be configurable?)
 * Note: this must be at least 3, to comply with the spec. */
#define ACAP_INHERITANCE_DEPTH 3

  typedef uint32_t dataset_key_t;

#define UNDETERMINED_DATASET_KEY (0)
#endif				/* CONFIG_H */
