/* util.c - various stupid little functions
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#define MKDIRHIER_C
#include <config.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include "util.h"

result_t
mkdirhier(char *path, const char *prefix, mode_t mode)
{
  result_t        result = OK;

  char           *here = NULL;

  I(path);

  VL(("Making path %s\n", path));

  if (prefix) {
    /* Start after the prefix, if appropriate */
    unsigned        len = strlen(prefix);

    if (!strncmp(path, prefix, len)) {
      here = strchr(path + len, '/');
    }
  }
  if (!here) {			/* nothing set here, so */
    here = strchr(path, '/');	/* start with the first '/' */
  }
  do {
    if (here != path) {		/* ignore leading '/' */
      if (here)
	*here = '\0';		/* terminate path at here */

      VL(("Making dir %s\n", path));

      if (mkdir(path, mode)) {	/* make the dir */
	result = errno;
	if (result == EEXIST)	/* this isn't an error */
	  result = OK;
      }
    }
    if (here) {
      *here = '/';		/* change it back */
      here++;
      here = strchr(here, '/');	/* and update it */
    }
  } while (result == OK && here);	/* keep going until error or complete */

  /* And make the final component (the one without a '/' at the end). */
  if (result == OK)
    VL(("Making dir %s\n", path));
  if (mkdir(path, mode)) {
    result = errno;
    if (result == EEXIST)
      result = OK;
  }
  return result;
}
